@@ -1,6 +1,6 @@
 package com.genymobile.scrcpy.wrappers;
 
-import com.genymobile.scrcpy.Ln;
+import com.genymobile.scrcpy.util.Ln;
 
 import android.annotation.TargetApi;
 import android.os.IInterface;
